# Introduction

`ambient-driver` is a prototype for running CI on a project, using
`ambient-run`.

# Smoke test

_Want:_ The `ambient-driver` tool can show it's runtime
configuration.

_Why:_ If this doesn't work, there's no hope of anything
else working, either.

_Who:_ Lars


~~~scenario
given an installed ambient-driver
given file .config/ambient-driver/config.yaml from config.yaml
when I run ambient-driver config
then stdout contains ""executor": "ambient-execute-plan""
~~~

~~~{#config.yaml .file .yaml}
executor: ambient-execute-plan
~~~

# Allows specifying a configuration file

_Want:_ The `ambient-driver` tool allows the user to specify
which configuration file to use.

_Why:_ This is very convenient when one wants to try things
and temporarily not use the default configuration file. This happens
both for experimentation and for testing.

_Who:_ Lars


~~~scenario
given an installed ambient-driver
given file special.yaml
when I run ambient-driver --config special.yaml config
then command is successful
~~~

~~~{#special.yaml .file .yaml}
executor: ambient-execute-plan
~~~

# Capture build log when running

_Want:_ The stdout/stderr output of the commands executed by
the CI run must be captured.

_Why:_ This is the primary way for the user to know what
happened during a run.

_Who:_ Lars.

Note that we can't use `.` for the current directory as the `source`
field in the project file due to how Subplot sets up a temporary data
directory for each scenario, and sets `TMPDIR` to that directory.
This would mean that `.` as `source` would create a tar archive in the
temporary directory that tries to add itself, which creates an
infinitely large tar archive.

~~~scenario
given an installed ambient-driver
given an Ambient VM image ambient.qcow2
given file .config/ambient-driver/config.yaml from config.yaml
given file hello.yaml
given a directory srcdir
when I run env AMBIENT_LOG=trace ambient-driver run --projects hello.yaml
then command is successful
~~~

~~~{#hello.yaml .file .yaml}
projects:
  hello:
    image: ambient.qcow2
    source: srcdir
    plan:
    - action: shell
      shell: |
        echo well hello there, friend
~~~

# Store run log in project directory

_Want:_ The run log should be stored persistently in the project state
directory, and there is a way to retrieve it later.

_Why:_ Run logs are often needed long after the run has happened.

_Who:_ Lars.

~~~scenario
given an installed ambient-driver
given an Ambient VM image ambient.qcow2
given file .config/ambient-driver/config.yaml from config.yaml
given file hello.yaml
given a directory srcdir
when I run env AMBIENT_LOG=debug  ambient-driver run --projects hello.yaml
when I run find .local/state/ambient-driver
when I run cat .local/state/ambient-driver/hello/run.log
then file .local/state/ambient-driver/hello/run.log contains "hello there, friend"
when I run ambient-driver log hello
then stdout contains "hello there, friend"
~~~


# Relative filenames in project files

_Want:_ When a project file has a relative filename, it's
relative to the directory containing the project file.

_Why:_ The user can then run `ambient-driver run` from
anywhere, not just the source directory.

_Who:_ Lars.

~~~scenario
given an installed ambient-driver
given file .config/ambient-driver/config.yaml from config.yaml
given a directory path/to/project/srcdir
given an Ambient VM image ambient.qcow2
when I run mv ambient.qcow2 path/to/project/
given file path/to/project/hello.yaml from hello.yaml
when I run env AMBIENT_LOG=trace ambient-driver run --projects path/to/project/hello.yaml
then command is successful
~~~

# Working directory in pre- and post-plan actions

_Want:_ When a pre- or post-plan action is executed, the
current working directory should be the source directory.

_Why:_ Many actions can only usefully be executed in the
source directory.

_Who:_ Lars.

Note that we can't use `.` for the current directory as the `source`
field in the project file due to how Subplot sets up a temporary data
directory for each scenario, and sets `TMPDIR` to that directory.
This would mean that `.` as `source` would create a tar archive in the
temporary directory that tries to add itself, which creates an
infinitely large tar archive.

~~~scenario
given an installed ambient-driver
given file .config/ambient-driver/config.yaml from config.yaml
given an Ambient VM image ambient.qcow2
given file cwd.yaml
given a directory path/to/project/srcdir
when I run env AMBIENT_LOG=trace ambient-driver run --projects cwd.yaml
then stderr matches regex INFO.*cwd:.*/path/to/project/srcdir
~~~

~~~{#cwd.yaml .file .yaml}
projects:
  pwd:
    image: ambient.qcow2
    source: path/to/project/srcdir
    pre_plan:
    - action: pwd
    post_plan:
    - action: pwd
    plan:
    - action: shell
      shell: |
        pwd
~~~

# Run CI only for some projects

_Want:_ When the projects file has more than one project, user
can choose only specific ones to run CI for.

_Why:_ User may be only interested in one project right now.

_Who:_ Lars

~~~scenario
given an installed ambient-driver
given file .config/ambient-driver/config.yaml from config.yaml
given an Ambient VM image ambient.qcow2
given file multi.yaml
given a directory foo
given a directory bar
when I run env AMBIENT_LOG=debug ambient-driver run --dry-run --projects multi.yaml foo
then stderr contains "project foo: NOT running CI"
then stderr doesn't contain "project bar:"
~~~

~~~{#multi.yaml .file .yaml}
projects:
  foo:
    image: ambient.qcow2
    source: foo
    plan:
    - action: shell
      shell: |
        echo foo project
  bar:
    image: ambient.qcow2
    source: bar
    plan:
    - action: shell
      shell: |
        echo bar project
~~~

# List names of projects

_Want:_ The user can list names of projects in a projects file.

_Why:_ This is handy for checking and scripting.

_Who:_ Lars

~~~scenario
given an installed ambient-driver
given file .config/ambient-driver/config.yaml from config.yaml
given an Ambient VM image ambient.qcow2
given file multi.yaml
given a directory foo
given a directory bar
when I run ambient-driver projects --oneline --projects multi.yaml
then stdout contains "{"projects":["bar","foo"]}"
~~~

# List names of actions

_Want:_ The user can list names of projects in a projects file.

_Why:_ This is handy for checking and scripting.

_Who:_ Lars

~~~scenario
given an installed ambient-driver
given file .config/ambient-driver/config.yaml from config.yaml
given an Ambient VM image ambient.qcow2
when I run ambient-driver actions
then stdout contains ""pre_actions": [\n"
then stdout contains ""actions": [\n"
then stdout contains ""post_actions": [\n"
~~~
# Cache persists between CI runs

_Want:_ Cache data is persisted between CI runs.

_Why:_ This allows incrementally building a project after
changes.

_Who:_ Lars

~~~scenario
given an installed ambient-driver
given file .config/ambient-driver/config.yaml from config.yaml
given an Ambient VM image ambient.qcow2
given file cache.yaml
given a directory srcdir

when I run ambient-driver run --projects cache.yaml
when I run ambient-driver log hello
then stdout contains "counter is now 1."

when I run ambient-driver run --projects cache.yaml
when I run ambient-driver log hello
then stdout contains "counter is now 2."

when I run ambient-driver run --projects cache.yaml
when I run ambient-driver log hello
then stdout contains "counter is now 3."
~~~

~~~{#cache.yaml .file .yaml}
projects:
  hello:
    image: ambient.qcow2
    source: srcdir
    plan:
    - action: shell
      shell: |
        cache=/workspace/cache
        counter="$cache/counter"
        if [ -e "$counter" ]; then
            n="$(expr "$(cat "$counter")" + 1)"
            echo "$n" > "$counter"
        else
            echo 1 > "$counter"
        fi
        echo "counter is now $(cat "$counter")."
        find "$cache" -ls
~~~
# Publish files via rsync

_Want:_ Artifacts can be published via rsync to a server.

_Why:_ This allows publishing many kinds of things.

_Who:_ Lars

~~~scenario
given an installed ambient-driver
given file .config/ambient-driver/config.yaml from config.yaml
given an Ambient VM image ambient.qcow2
given file rsync.yaml
given a directory srcdir
given file srcdir/data.dat from rsync.yaml

given a directory serverdir
when I run env AMBIENT_LOG=trace ambient-driver run --projects rsync.yaml --target serverdir
then file serverdir/hello.txt contains "hello, world"
~~~

~~~{#rsync.yaml .file .yaml}
projects:
  hello:
    image: ambient.qcow2
    source: srcdir
    plan:
    - action: shell
      shell: |
        echo hello, world > /workspace/artifacts/hello.txt
    post_plan:
    - action: rsync
~~~
